package ABB_CC_Testcase "ABB Test Case Model Definition" 
  extends Modelica.Icons.Library;
  
  annotation (
    Coordsys(
      extent=[0, 0; 404, 368], 
      grid=[2, 2], 
      component=[20, 20]), 
    Window(
      x=0.45, 
      y=0.01, 
      width=0.4, 
      height=0.54, 
      library=1, 
      autolayout=1), 
    Documentation(info=" <HTML>
		<P>This Modelica package implements a test case that illustrates voltage stability 
			in power systems. It has been constructed within the framework of the 
			EU-sponsored Research program Control and Computation. Additional theoretical 
			background can be found in the textbooks on computer modelling power systems in 
			general (e.g. [1]) on voltage stability (e.g. [2,3]).


It can be used as a small-scale example for the development and testing of emergency control 
methods against voltage collapse. The model contains nonlinearity as well as hybrid behaviour and
can also be used to test algorithms. The test case will later be extended to medium and large
scale power systems and can thus also be used to study large-scale aspects of methods being evaluated.
</P>
		<P>Questions regarding the test case can be answered by Mats Larsson.</P>
		<P><B>REFERENCES</B></P>
		<P>[1] J. Machowski, J.W. Bialek, and J.R. Bumby, <I>Power System Dynamics and 
				Stability</I>, Number ISBN 0-471-97174. Wiley, 1993.</P>
		<P>[2] T. van Cutsem and C. Vournas, <I>Voltage Stability of Electric Power Systems</I>, 
			Number ISBN 0-7923-8139-4. Kluwer Academic Publishers, 1998.</P>
		<P>[3] M. Larsson, <A href=\"http://www.iea.lth.se/~ielmatsl/Publications/thesis.html\">Coordinated 
				Voltage Control in Power Systems</A>, PhD Thesis, Lund University, Sweden, 
			2000.</P>
		<P><STRONG>VERSION HISTORY</STRONG></P>
		<UL>
			<LI>
				v1.0 - first version of ABB Test Case Library, Feb 08, 2002.</LI></UL>
		<P><STRONG>AUTHOR</STRONG></P>
		<P>Mats Larsson, Corporate Research, ABB Schweiz AG,&nbsp;5405 Baden-Dttwil, 
			Switzerland.</P>
		<P>&nbsp;</P>
		<P>&nbsp;</P>
		<P>&nbsp;</P>

</html>
"));
  package Components 
    extends Modelica.Icons.Library2;
    
    model Slack "Slack Node" 
      extends Basic.OnePin;
      
      parameter Real V0=1;
      parameter Real theta0=0;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.21, 
          y=0.19, 
          width=0.6, 
          height=0.6), 
        Icon(
          Text(extent=[-40, -20; 80, 20], string="inf"), 
          Line(points=[-100, 0; -40, 0], style(
              color=0, 
              pattern=1, 
              thickness=1, 
              arrow=0)), 
          Ellipse(extent=[-40, 60; 80, -60], style(
              color=0, 
              pattern=1, 
              thickness=1, 
              arrow=0)), 
          Text(extent=[-60, -68; 100, -106], string="%name")), 
        Documentation(info="The slack node model acts as slack bus in initialization or load-flow 
calculations and as an infinite bus during dynamic simulation. 
In addition it defines the reference for the phasor calculation.



"), 
        Diagram);
    equation 
      1 + T.va = V0*cos(theta0);
      T.vb = V0*sin(theta0);
    end Slack;
    
    model Gen3 "3rd Order Detailed Generator" 
      extends Basic.OnePin;
      parameter Real H=3 "Inertia Constant";
      parameter Real D=0 "Damping Coefficient";
      
      parameter Real ra=0 "Armature Resistance";
      parameter Real xd=0.8948 "Direct Axis Synchronous Reactance";
      parameter Real xq=0.84 "Quadrature Axis Synchronous Reactance";
      parameter Real xdp=0.30 "Direct Axis Transient Reactance";
      parameter Real Td0p=7 "Open-circuit Direct Axis Transient Time Constant";
      
      parameter Real Ef0=1.72250079258051;
      parameter Real Efmax=2.2;
      
      parameter Real Pm0=0.352;
      parameter Real Pmmax=1;
      parameter Real wref=1;
      
      input Real Vref;
      
      Real Eqp(start=1);
      Real w(start=1) "Angular Speed";
      Real delta "Rotor Angle";
      
      Real id;
      Real iq "dq Armature Current";
      Real vd;
      Real vq "dq Armature Voltage";
      Real Pe;
      
      Real Efd;
      Real dEfd;
      Real Pm;
      Real dPm;
      Real V;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.33, 
          y=0.01, 
          width=0.67, 
          height=0.98), 
        Icon(
          Line(points=[-100, 0; -40, 0], style(
              color=0, 
              pattern=1, 
              thickness=1, 
              arrow=0)), 
          Ellipse(extent=[-40, 60; 80, -60], style(
              color=0, 
              pattern=1, 
              thickness=1, 
              arrow=0)), 
          Text(extent=[-60, -68; 100, -106], string="%name")), 
        Documentation(info="The 3rd order detailed generator model corresponds to Model 3 in [1, pp 348],
and extends the DetGen class. It adds a single transient EMF source in
the quatradure axis and the field voltage input. This model neglects the effect
of damper windings and the damping produced by rotor eddy-currents.
The effect of this damping is usually included in the damping coefficient D.

Each generator has it's own dq coordinate system, and its stator equations
must therefore be related to the network (global) coordinate system using the
so called Kron's transformations [1, pp. 90].



---
[1] J. Machowski, J.W. Bialek, and J.R. Bumby, Power System Dynamics and
Stability, Number ISBN 0-471-97174. Wiley, 1993.
"));
    equation 
      V = sqrt((1 + T.va)*(1 + T.va) + T.vb*T.vb);
      
      //   swing equations
      der(w) = 1/(2*H)*(Pm - Pe - D*(w - 1));
      der(delta) = 2*Modelica.Constants.PI*50*(w - 1);
      
      // Transient EMF equation
      Td0p*der(Eqp) = Efd - Eqp + id*(xd - xdp);
      
      // stator equations
      vd = -ra*id - xq*iq;
      vq = Eqp + xdp*id - ra*iq;
      
      // electrical power
      Pe = Eqp*iq + (xdp - xq)*id*iq;
      
      // coordinate change
      -T.ia = -sin(delta)*id + cos(delta)*iq;
      -T.ib = cos(delta)*id + sin(delta)*iq;
      1 + T.va = -sin(delta)*vd + cos(delta)*vq;
      T.vb = cos(delta)*vd + sin(delta)*vq;
      
      // generator voltage control
      2.5*der(dEfd) + dEfd = 50*(Vref - V);
      Efd = min(Efmax, dEfd + Ef0);
      // governor control
      10*der(dPm) + dPm = 1.5*(wref - w);
      Pm = min(Pmmax, dPm + Pm0);
    end Gen3;
    
    model Load "Dynamic exponential recovery load" 
      extends Basic.OnePin;
      parameter Real xp0=0 "Initial Value for xp";
      parameter Real xq0=0 "Initial Value for xq";
      parameter Real P0=.1 "Rated Active Power Load";
      parameter Real Q0=.02 "Rated Reactive Power Load";
      parameter Real as=0 "Steady-state active power voltage dependency";
      parameter Real at=2 "Transient active power voltage dependency";
      parameter Real bs=0 "Steady-state reactive power voltage dependency";
      parameter Real bt=2 "Transient reactive power voltage dependency";
      parameter Real Tp=60 "Active power recovery time constant";
      parameter Real Tq=60 "Reactive power recovery time constant";
      parameter Real ShedStep=0.05;
      discrete input Integer step(min=0, max=3) "Load Shedding Constant";
      Real xp(
        min=-1, 
        max=1, 
        start=xp0, 
        fixed=true) "Internal Load State";
      Real xq(
        min=-1, 
        max=1, 
        start=xq0, 
        fixed=true) "Internal Load State";
      Real V(min=-1.5, max=1.5);
      Real Pl(min=0, max=1);
      Real Ql(min=-0.5, max=1);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Line(points=[-40, -40; -20, -40; -20, -60; -6, -50; 6, -46; 24, -44; 
                40, -44]), 
          Polygon(points=[40, -2; -20, 18; -20, -22; 40, -2], style(
              color=0, 
              gradient=3, 
              fillColor=0)), 
          Line(points=[-100, 0; -20, 0], style(color=0)), 
          Text(extent=[-60, 60; 60, 20], string="%P0 + j %Q0")), 
        Window(
          x=0.12, 
          y=0.04, 
          width=0.7, 
          height=0.89), 
        Documentation(info="Dynamic exponential recovery load according to Karlsson & Hill [1].

The load powers are given by:

  der(xp) = P0*(V^(as) - V^at) - xp/Tp;
  Pl = (xp/Tp + P0*V^at);
  der(xq) = Q0*(V^(bs) - V^bt) - xq/Tq;
  Ql = (xq/Tq + Q0*V^bt);

where xp is a continuous dynamic state that can be interpreted as a
measure of the energy deficit in the load and Ps(V) = P0*V^as 
is the steady-state and Pt(V)=P0*V^at the transient voltage dependency.
Pl is the actual active load power and Tp is the active power recovery 
time constant. 

For the reactive load power, a similar model is used with corresponding 
characteristics x_q, Qs(V)=Q0 V^bs, Qt(V) = Q0 V^bt and time constant Tq.

---
[1] D. Karlsson and D.J. Hill, \"Modelling and identification of nonlinear 
dynamic loads in power systems\", IEEE Transactions on Power Systems, 
vol. 9, no. 1, pp. 157-163, February 1994.

"));
    equation 
      Pl = (1 - step*ShedStep)*(xp/Tp + P0*V^at);
      Ql = (1 - step*ShedStep)*(xq/Tq + Q0*V^bt);
      der(xp) = P0*(V^as - V^at) - xp/Tp;
      der(xq) = Q0*(V^bs - V^bt) - xq/Tq;
      
      T.ia = (T.vb*Ql + Pl + Pl*T.va)/(1 + 2*T.va + T.va*T.va + T.vb*T.vb);
      T.ib = (Pl*T.vb - Ql - T.va*Ql)/(1 + 2*T.va + T.va*T.va + T.vb*T.vb);
      
      V = sqrt((1 + T.va)*(1 + T.va) + T.vb*T.vb);
      
    end Load;
    
    model GenQSS "Quasi-Steady State Model of a generator" 
      extends Basic.OnePin;
      // typical limits for terminal variables
      // T.ia(min=-1.05, max=1.05), T.ib(min=-1.05, max=1.05)
      // T.va(min=-0.5, max=0.5  ), T.vb(min=-0.5,  max=0.5)
      parameter Real H(
        min=0.1, 
        max=20) = 3 "Inertia Constant";
      parameter Real D(
        min=0, 
        max=20) = 0 "Damping Coefficient";
      parameter Real ra(
        min=0.0, 
        max=0.02) = 0 "Armature Resistance";
      parameter Real xd(
        min=0.6, 
        max=2.4) = 0.8948 "Direct Axis Synchronous Reactance";
      parameter Real xq(
        min=0.4, 
        max=2.4) = 0.84 "Quadrature Axis Synchronous Reactance";
      parameter Real xdp(
        min=0.15, 
        max=1) = 0.30 "Direct Axis Transient Reactance";
      parameter Real Td0p(
        min=1.5, 
        max=10) = 7 "Open-circuit Direct Axis Transient Time Constant";
      
      parameter Real Ef0(
        min=1.5, 
        max=9) = 1.72250079258051;
      parameter Real Efmax=2.2;
      
      parameter Real Pm0=0.352;
      parameter Real Pmmax=1;
      parameter Real wref=1;
      
      Real Eqp(
        start=1, 
        min=0, 
        max=2);
      Real w(
        start=1, 
        min=0.8, 
        max=1.2) "Angular Speed";
      Real delta(min=-Modelica.Constants.PI/2, max=Modelica.Constants.PI/2);
      
      Real id(min=-1.2, max=1.2);
      Real iq(min=-1.2, max=1.2);
      Real vd(min=-1.2, max=1.2);
      Real vq(min=-1.2, max=1.2);
      Real Pe(min=0, max=1.05);
      
      Real Efd(min=0, max=7);
      Real Pm(min=0, max=1.05);
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Ellipse(extent=[-40, 60; 80, -60], style(
              color=0, 
              pattern=1, 
              thickness=1, 
              arrow=0)), 
          Line(points=[-100, 0; -40, 0], style(
              color=0, 
              pattern=1, 
              thickness=1, 
              arrow=0)), 
          Text(extent=[-60, -68; 100, -106], string="%name")), 
        Window(
          x=0.18, 
          y=0, 
          width=0.7, 
          height=1), 
        Documentation(info="This model neglects the electromechanical dynamics of the
generator. 
"));
    equation 
      //   swing equations
      0 = 1/(2*H)*(Pm - Pe - D*(w - 1));
      0 = 2*Modelica.Constants.PI*(w - 1);
      
      // Transient EMF equation
      Eqp = (Efd + id*(xd - xdp));
      
      // stator equations
      vd = -ra*id - xq*iq;
      vq = Eqp + xdp*id - ra*iq;
      
      // electrical power
      Pe = Eqp*iq + (xdp - xq)*id*iq;
      
      // coordinate change
      -T.ia = -sin(delta)*id + cos(delta)*iq;
      -T.ib = cos(delta)*id + sin(delta)*iq;
      1 + T.va = -sin(delta)*vd + cos(delta)*vq;
      T.vb = cos(delta)*vd + sin(delta)*vq;
    end GenQSS;
    
    model Transformer "Transformer with Voltage Controller " 
      extends ImpTransformer;
      
      parameter Real stepsize=0.02;
      Integer tappos;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.06, 
          y=0.04, 
          width=0.61, 
          height=0.91), 
        Documentation(info="The tap changer controller is modelled as a state automata.
"));
    equation 
      Tr.n = 1 + stepsize*tappos;
    end Transformer;
    
    class Varimp 
      extends Basic.TwoPin;
      
      parameter Real R=0.0 "Resistance";
      
      input Real X "Reactance";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Rectangle(extent=[-60, 20; 60, -20], style(color=0)), 
          Line(points=[-60, 0; -100, 0], style(color=0)), 
          Line(points=[100, 0; 60, 0], style(color=0)), 
          Polygon(points=[4, 64; -50, -20; 20, 2; -24, -66; 50, 26; -30, -10; 4
                , 64], style(
              color=6, 
              fillColor=6, 
              fillPattern=1)), 
          Polygon(points=[24, 76; -30, -8; 38, 12; -4, -54; 70, 38; -10, 2; 24
                , 76], style(
              color=6, 
              gradient=2, 
              fillColor=1, 
              fillPattern=1)), 
          Text(extent=[34, 58; -46, 98], string="%name")), 
        Window(
          x=0.19, 
          y=0.11, 
          width=0.6, 
          height=0.6));
    equation 
      [T1.va - T2.va; T1.vb - T2.vb] = [R, -X; X, R]*[T1.ia; T1.ib];
      [T1.ia; T1.ib] + [T2.ia; T2.ib] = [0; 0];
    end Varimp;
    
    class Capacitor 
      extends Basic.OnePin;
      
      parameter Real G=0;
      parameter Real B=0.1;
      discrete input Integer step(min=0, max=3);
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Line(points=[-20, 40; -20, -40], style(color=0)), 
          Line(points=[0, 40; 0, -40], style(color=0)), 
          Text(extent=[32, 46; -48, 86], string="%name"), 
          Line(points=[-100, 0; -20, 0], style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.44, 
          height=0.65));
    equation 
      [T.ia; T.ib] = step*[G, -B; B, G]*[1 + T.va; T.vb];
      
    end Capacitor;
    
    model IdealTransformer 
      extends Basic.TwoPin;
      Real n(start=1);
      Modelica.Blocks.Interfaces.InPort inPort(final n=1) annotation (extent=[-
            10, -90; 10, -70], rotation=90);
      annotation (
        Diagram, 
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.01, 
          y=0.1, 
          width=0.6, 
          height=0.6), 
        Icon(
          Ellipse(extent=[-70, 40; 12, -40], style(color=0)), 
          Ellipse(extent=[-12, 40; 70, -40], style(color=0)), 
          Line(points=[70, 0; 100, 0], style(color=0)), 
          Line(points=[-100, 0; -70, 0], style(color=0)), 
          Text(extent=[-60, 80; 60, 40], string="%name"), 
          Text(extent=[-10, -78; -50, -38], string="1"), 
          Text(extent=[20, -76; -20, -36], string=":"), 
          Text(extent=[50, -76; 10, -36], string="n")));
    equation 
      (1 + T1.va)*n = 1 + T2.va;
      T1.vb*n = T2.vb;
      T1.ia = -T2.ia*n;
      T1.ib = -T2.ib*n;
      n = inPort.signal[1];
    end IdealTransformer;
    
    partial model ImpTransformer 
      extends Basic.TwoPin;
      parameter Real R=0.0 "Leakage Resistance";
      parameter Real X=0.1 "Leakage Reactance";
      annotation (
        Icon(
          Ellipse(extent=[-70, 40; 12, -40], style(color=0)), 
          Ellipse(extent=[-12, 40; 70, -40], style(color=0)), 
          Line(points=[70, 0; 100, 0], style(color=0)), 
          Line(points=[-100, 0; -70, 0], style(color=0)), 
          Text(extent=[-60, -40; 60, -80], string="%name")), 
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6));
      IdealTransformer Tr annotation (extent=[20, -20; 60, 20]);
      Impedance Imp(R=R, X=X) annotation (extent=[-60, -20; -20, 20]);
    equation 
      connect(Imp.T1, T1) annotation (points=[-60, 0; -100, 0]);
      connect(Imp.T2, Tr.T1) annotation (points=[-20, 0; 20, 0]);
      connect(Tr.T2, T2) annotation (points=[60, 0; 100, 0]);
    end ImpTransformer;
    annotation (
      Coordsys(
        extent=[0, 0; 606, 355], 
        grid=[2, 2], 
        component=[20, 20]), 
      Window(
        x=0.4, 
        y=0.4, 
        width=0.6, 
        height=0.6, 
        library=1, 
        autolayout=1), 
      Icon);
    package Basic 
      extends Modelica.Icons.Library;
      connector Pin "Electrical connector for ObjectStab models" 
        Real va;
        Real vb;
        flow Real ia;
        flow Real ib;
        
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100], 
            grid=[2, 2], 
            component=[20, 20]), 
          Icon(Rectangle(extent=[-20, 20; 20, -20], style(fillColor=73))), 
          Window(
            x=0.4, 
            y=0.4, 
            width=0.6, 
            height=0.6));
      end Pin;
      
      partial model OnePin 
        "Shell model for ObjectStab models with one electrical connector " 
        Pin T;
      end OnePin;
      
      partial model OnePinCenter 
        "Shell model for models with one connector in the center of the model" 
        Pin T annotation (extent=[-20, -20; 20, 20]);
        annotation (Coordsys(
            extent=[-100, -100; 100, 100], 
            grid=[2, 2], 
            component=[20, 20]), Window(
            x=0.4, 
            y=0.4, 
            width=0.6, 
            height=0.6));
      end OnePinCenter;
      
      partial model TwoPin 
        "Shell model for ObjectStab models with two electrical connectors " 
        Pin T1;
        Pin T2;
      equation 
        
      end TwoPin;
      annotation (Coordsys(
          extent=[0, 0; 606, 355], 
          grid=[2, 2], 
          component=[20, 20]), Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6, 
          library=1, 
          autolayout=1));
    end Basic;
    
    model Bus "Busbar model" 
      extends Basic.OnePinCenter;
      Real V;
      Real theta;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6), 
        Diagram, 
        Icon(Rectangle(extent=[-12, 80; 10, -80], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), Text(extent=[80, -122; -80, -82], string="%name"
            )));
    equation 
      T.ia = 0;
      T.ib = 0;
      V = sqrt((1 + T.va)*(1 + T.va) + T.vb*T.vb);
      theta = Modelica.Math.atan2(T.vb, (1 + T.va));
    end Bus;
    
    model Impedance "Impedance model" 
      extends Basic.TwoPin;
      
      parameter Real R=0.0 "Resistance";
      parameter Real X=0.1 "Reactance";
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6), 
        Icon(
          Rectangle(extent=[-60, 20; 60, -20], style(color=0)), 
          Line(points=[-60, 0; -100, 0], style(color=0)), 
          Line(points=[100, 0; 60, 0], style(color=0)), 
          Text(extent=[40, 20; -40, 60], string="%name")), 
        Documentation(info="The impedance model is governed by the equations:

           R+jX
           ----
V1, I1  ---    --- V2, I2
    ->     ----        <-


V1 - V2 = (R+jX) * I1
I1 + I2 = 0:


For numerical reasons, R and X may not both be set to zero.
"), 
        Diagram(
          Rectangle(extent=[-60, 20; 60, -20], style(color=0)), 
          Line(points=[-60, 0; -100, 0], style(color=0)), 
          Line(points=[100, 0; 60, 0], style(color=0)), 
          Text(extent=[40, 20; -40, 60], string="R+jX"), 
          Line(points=[-92, -10; -66, -10], style(arrow=1)), 
          Text(extent=[-60, -40; -100, -20], string="V1, I1"), 
          Text(extent=[100, -40; 60, -20], string="I2,V2"), 
          Line(points=[94, -12; 66, -12], style(arrow=1))));
    equation 
      [T1.va - T2.va; T1.vb - T2.vb] = [R, -X; X, R]*[T1.ia; T1.ib];
      [T1.ia; T1.ib] + [T2.ia; T2.ib] = [0; 0];
    end Impedance;
    
    model Pilink "Pilink Line Model" 
      extends Basic.TwoPin;
      
      parameter Real R=0.0 "Resistance";
      parameter Real X=0.1 "Reactance";
      parameter Real B=0.1 "Shunt Susceptance";
      parameter Real G=0.0 "Shunt Conductance";
      annotation (Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), Icon(
          Line(points=[100, 0; 60, 0], style(color=0)), 
          Rectangle(extent=[-60, 20; 60, -20], style(color=0)), 
          Line(points=[-60, 0; -100, 0], style(color=0)), 
          Text(extent=[100, 20; -100, 60], string="%name")));
    equation 
      [T1.ia; T1.ib] = [G, -B; B, G]/2*[1 + T1.va; T1.vb] + [R, X; -X, R]/(R^2
         + X^2)*[T1.va - T2.va; T1.vb - T2.vb];
      [T2.ia; T2.ib] = [G, -B; B, G]/2*[1 + T2.va; T2.vb] - [R, X; -X, R]/(R^2
         + X^2)*[T1.va - T2.va; T1.vb - T2.vb];
      
    end Pilink;
    
    model GenQSS2 "Quasi-Steady State Model of a generator" 
      extends Basic.OnePin;
      parameter Real H(
        min=0.1, 
        max=20) = 3 "Inertia Constant";
      parameter Real D(
        min=0, 
        max=20) = 0 "Damping Coefficient";
      parameter Real ra(
        min=0.0, 
        max=0.02) = 0 "Armature Resistance";
      parameter Real xd(
        min=0.6, 
        max=2.4) = 0.8948 "Direct Axis Synchronous Reactance";
      parameter Real xq(
        min=0.4, 
        max=2.4) = 0.84 "Quadrature Axis Synchronous Reactance";
      parameter Real xdp(
        min=0.15, 
        max=1) = 0.30 "Direct Axis Transient Reactance";
      parameter Real Td0p(
        min=1.5, 
        max=10) = 7 "Open-circuit Direct Axis Transient Time Constant";
      
      Real Pg(min=-0.2, max=1.05);
      Real Qg(min=-0.5, max=0.5);
      
      Real Efd(min=0, max=7);
      Real V(min=0.5, max=1.5);
      parameter Real Pm0=0.352;
      
      annotation (
        Icon(
          Ellipse(extent=[-40, 60; 80, -60], style(
              color=0, 
              pattern=1, 
              thickness=1, 
              arrow=0)), 
          Line(points=[-100, 0; -40, 0], style(
              color=0, 
              pattern=1, 
              thickness=1, 
              arrow=0)), 
          Text(extent=[-60, -68; 100, -106], string="%name")), 
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6));
    equation 
      V = sqrt((1 + T.va)*(1 + T.va) + T.vb*T.vb);
      Efd = (V^4 + V^2*Qg*xq + Qg*xd*V^2 + Qg^2*xd*xq + Pg^2*xq*xd)/(sqrt(Pg^2*
        xq^2 + V^4 + 2*V^2*Qg*xq + Qg^2*xq^2)*V);
      Pg = Pm0;
      Pg = -((1 + T.va)*T.ia + T.vb*T.ib);
      Qg = -(T.vb*T.ia - (1 + T.va)*T.ib);
    end GenQSS2;
    
  end Components;
  
  package Fournode "The small scale four node power system model" 
    extends Modelica.Icons.Library2;
    
    model OpenLoop "Test case power system without primary controls" 
      parameter Real L3X_1=0.5;
      parameter Real L3X_2=1.5;
      
      input Integer CapStep;
      input Integer LoadStep;
      input Integer TrStep(start=1);
      input Real G1Efd;
      input Boolean faulted;
      
      output Real V2;
      output Real V3;
      output Real V4;
      parameter Boolean SimulinkSafe=true;
    protected 
      constant Real Ef0=1.55;
    public 
      Components.Transformer T1(X=0.0031) annotation (extent=[-20, -52; 20, -12
            ], rotation=270);
      Components.Capacitor C3 annotation (extent=[-60, -62; -20, -20], rotation
          =270);
      Components.Varimp L3 annotation (extent=[-60, 20; -20, 60], rotation=90);
      Components.Bus Bus1 annotation (extent=[-80, 60; -40, 100]);
      Components.Bus Bus2 annotation (extent=[20, 60; 60, 100]);
      Components.Bus Bus3 annotation (extent=[-60, -20; 40, 20], rotation=90);
      Components.Bus Bus4 annotation (extent=[-20, -82; 20, -42], rotation=90);
      Components.Slack Ginf(V0=1.03) annotation (extent=[-100, 60; -60, 100], 
          rotation=180);
      Components.Impedance L1(X=0.75) annotation (extent=[-30, 60; 10, 100]);
      Components.Impedance L2(X=0.7) annotation (extent=[0, 20; 40, 60], 
          rotation=90);
      Components.Load Load(
        P0=1, 
        Q0=0.2, 
        as=0, 
        at=2, 
        bs=1, 
        bt=2) annotation (extent=[-20, -102; 20, -62], rotation=270);
      replaceable Components.GenQSS2 G1(
        xd=2.38, 
        xq=2.27, 
        xdp=0.91, 
        Td0p=9.6) annotation (extent=[40, 60; 80, 100]);
    equation 
      connect(Bus4.T, T1.T2) annotation (points=[0, -62; -3.55271e-015, -52]);
      connect(C3.T, Bus3.T) annotation (points=[-40, -20; -40, 0; -10, 0]);
      connect(T1.T1, Bus3.T) annotation (points=[3.55271e-015, -12; 0, 0; -10, 
            0]);
      connect(L3.T1, Bus3.T) annotation (points=[-40, 20; -40, 0; -10, 0]);
      connect(L2.T1, Bus3.T) annotation (points=[20, 20; 20, 0; -10, 0]);
      annotation (
        Coordsys(
          extent=[-100, -102; 80, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.4, 
          y=0.01, 
          width=0.55, 
          height=0.98), 
        Diagram(Line(points=[0, -64; 0, -62])), 
        Icon(Rectangle(extent=[-80, 80; 60, -80], style(fillColor=72)), Text(
              extent=[-80, 60; 60, 20], string="OpenLoop")), 
        Documentation(info="Test case power system without primary controls is a nonlinear 
system with pure continuous dynamics. 

The open loop power system has four control inputs:
CapStep 
  Nr of capacitor steps connected at bus 3
  can assume the values [0,1,2,3]. Each step corresponds
  to 0.1 p.u. of compensation.
 
LoadStep
  The nr of load shedding steps used at bus 4
  can assume the values [0,1,2,3]. Each step corresponds
TrStep
   The position of the tap changer on transformer T1
G1Efd
   The field voltage of generator G1
 
The test case has one disturbance input:
  
faulted
   Boolean variable indicating whether or not line L3 is faulted.
   The fault is modelled by the change of the impedance of the line
   from 1 p.u. to 1.5 p.u. 
  
The open loop power system has three outputs:
V2,V3,V4
   The voltage at buses 2,3 and 4 respectively.
   The aim of emergency control is to keep the voltage of
   V4 within the limits [0.9 p.u. and 1.1 p.u.] despite
   the fault input becoming true. The control of voltages
   V2 and V3 is of secondary importance.
Recommended initial input values are:
CapStep=2, LoadStep=0, TrStep=1, and faulted=false (0)
"));
      connect(Bus1.T, Ginf.T) annotation (points=[-60, 80; -60, 80]);
      connect(L1.T2, Bus2.T) annotation (points=[10, 80; 40, 80]);
      connect(Bus2.T, L2.T2) annotation (points=[40, 80; 40, 72; 20, 72; 20, 60
            ]);
      connect(L3.T2, Ginf.T) annotation (points=[-40, 60; -40, 68; -40, 70; -60
            , 70; -60, 80]);
      connect(Ginf.T, L1.T1) annotation (points=[-60, 80; -36, 80; -30, 80]);
      connect(Load.T, Bus4.T) annotation (points=[3.55271e-015, -62; 0, -62]);
      connect(Bus2.T, G1.T) annotation (points=[40, 80; 40, 80]);
      
      if SimulinkSafe then
        G1.Efd = if noEvent(abs(G1Efd) > 1e-5) then G1Efd else Ef0;
      else
        G1.Efd = G1Efd;
      end if;
      C3.step = CapStep;
      Load.step = LoadStep;
      T1.tappos = TrStep;
      
      V2 = Bus2.V;
      V3 = Bus3.V;
      V4 = Bus4.V;
      L3.X = if faulted then L3X_2 else L3X_1;
    end OpenLoop;
    annotation (Coordsys(
        extent=[0, 0; 442, 394], 
        grid=[2, 2], 
        component=[20, 20]), Window(
        x=0.14, 
        y=0.05, 
        width=0.44, 
        height=0.65, 
        library=1, 
        autolayout=1));
    model PrimaryController "Primary Control System" 
      parameter Boolean idle0=false;
      parameter Boolean wait0=true;
      parameter Boolean action0=false;
      parameter Integer tappos0=1;
      parameter Real timer0=0;
      parameter Boolean up_limit0=false;
      
      input Real TapVref(start=1);
      input Real V2(start=1);
      input Real V3(start=1);
      input Real V4(start=1);
      
      Boolean idle(start=idle0, fixed=true);
      Boolean wait(start=wait0, fixed=true);
      Boolean action(start=action0, fixed=true);
      Integer tappos(start=tappos0, fixed=true);
      Real timer(start=timer0, fixed=true);
      Boolean up_limit(start=up_limit0, fixed=true);
      
      Boolean toohigh;
      Boolean toolow;
      
      output Real G1Efd;
      output Integer T1tappos;
      
      parameter Real G1Vref=1.03;
      parameter Real Ef0(
        min=1.5, 
        max=9) = 1.55;
      parameter Real Efmax=2.2;
      parameter Real TapDelay=30;
      parameter Real MechDelay=1;
      parameter Real TapDB=0.01*3;
      parameter Real MaxTap=10;
      parameter Real MinTap=-10;
      
      Real Efd(start=1.55);
      parameter Boolean SimulinkSafe=true;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Rectangle(extent=[-80, 80; 60, -80], style(fillColor=72)), 
          Text(extent=[-80, 60; 60, 20], string="Fournode"), 
          Text(extent=[-74, 14; 54, -34], string="Controller")), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.47, 
          height=0.86), 
        Documentation(info="This model models the primary control systems of the 
four node power system. 

It contains a tap changer controller for transformer T1
modelled as a state machine and an automatic voltage regulator
with filed voltage limiter modelled by a state machina and a linear 
control law.


The special equations activated by the flag SimulinkSafe
flag are jus fixes that must be applied to make the model simulable with
Simulink. If you remodel this system with other tools,
use the original equations (assume that SimulinkSafe=false)
"));
    equation 
      // generator voltage control
      if SimulinkSafe then
        0.01*der(Efd) + Efd = if up_limit then Efmax else Ef0 + (G1Vref - V2)*
          50;
      else
        Efd = if up_limit then Efmax else Ef0 + (G1Vref - V2)*50;
      end if;
      
      T1tappos = pre(tappos);
      G1Efd = Efd;
      // tap changer control - state automata !
      toohigh = (V4 - TapVref) > TapDB/2;
      toolow = (V4 - TapVref) < -TapDB/2;
      
      idle = (pre(idle) or pre(wait)) and not (toohigh or toolow) or (pre(
        action) and ((time - timer) > TapDelay + MechDelay));
      wait = (pre(idle) and (toohigh or toolow)) or pre(wait) and ((toolow or 
        toohigh) and ((time - pre(timer)) < TapDelay));
      action = (pre(wait) and (time - timer > TapDelay)) or pre(action) and ((
        time - timer) < TapDelay + MechDelay);
      up_limit = if not (pre(up_limit)) then (Efd > Efmax) else not (V2 > 
        G1Vref);
      when wait and not pre(wait) and not initial() then
        timer = time;
      end when;
      when pre(action) and not action then
        if toolow and (pre(tappos) < MaxTap) then
          tappos = pre(tappos) + 1;
        elseif toohigh and (pre(T1tappos) > MinTap) then
          tappos = pre(tappos) - 1;
        else
          tappos = pre(tappos);
        end if;
      end when;
      
    end PrimaryController;
    
    model PrimaryControlledSystem 
      "Test case power system with example primary controls applied" 
      input Integer CapStep;
      input Integer LoadStep;
      input Real TapVref;
      input Boolean faulted;
      output Real V2;
      output Real V3;
      output Real V4;
      output Real G1Efd;
      parameter Boolean SimulinkSafe=false;
      
      // auxilliary  outputs 
      output Real U1d;
      output Real U1q;
      output Real U2d;
      output Real U2q;
      output Real U3d;
      output Real U3q;
      output Real U4d;
      output Real U4q;
      
      output Real I1d;
      output Real I1q;
      output Real I2d;
      output Real I2q;
      output Real I3d;
      output Real I3q;
      output Real I4d;
      output Real I4q;
      output Real I5d;
      output Real I5q;
      output Real Icd;
      output Real Icq;
      
      output Real U2m;
      output Real U4m;
      output Real U4m2;
      output Real Efd;
      output Real P5l;
      output Real Q5l;
      
      output Integer T1_tappos;
      output Real T1_timer;
      output Real Load_xp;
      output Real Load_xq;
      output Boolean T1_idle;
      output Boolean T1_wait;
      output Boolean T1_action;
      output Boolean G1_up_limit;
      OpenLoop System(SimulinkSafe=SimulinkSafe) annotation (extent=[-70, 30; -
            10, 90]);
      PrimaryController Controller(Ef0=1.54, SimulinkSafe=SimulinkSafe) 
        annotation (extent=[10, 30; 70, 90]);
    equation 
      V2 = System.V2;
      V3 = System.V3;
      V4 = System.V4;
      G1Efd = Controller.G1Efd;
      Controller.TapVref = TapVref;
      System.LoadStep = LoadStep;
      System.CapStep = CapStep;
      Controller.V2 = System.V2;
      Controller.V3 = System.V3;
      Controller.V4 = System.V4;
      System.G1Efd = Controller.G1Efd;
      System.TrStep = Controller.T1tappos;
      System.faulted = faulted;
      U1d = System.Bus1.T.va;
      U1q = System.Bus1.T.vb;
      U2d = System.Bus2.T.va;
      U2q = System.Bus2.T.vb;
      U3d = System.Bus3.T.va;
      U3q = System.Bus3.T.vb;
      U4d = System.Bus4.T.va;
      U4q = System.Bus4.T.vb;
      
      I1d = System.Ginf.T.ia;
      I1q = System.Ginf.T.ib;
      I2d = System.G1.T.ia;
      I2q = System.G1.T.ib;
      I3d = System.T1.T1.ia;
      I3q = System.T1.T1.ib;
      I4d = System.T1.T2.ia;
      I4q = System.T1.T2.ib;
      I5d = System.Load.T.ia;
      I5q = System.Load.T.ib;
      Icd = System.C3.T.ia;
      Icq = System.C3.T.ib;
      
      U2m = System.Bus2.V;
      U4m = System.Bus4.V;
      U4m2 = System.Bus4.V^2;
      Efd = System.G1.Efd;
      P5l = System.Load.Pl;
      Q5l = System.Load.Ql;
      
      T1_tappos = Controller.tappos;
      T1_timer = Controller.timer;
      Load_xp = System.Load.xp;
      Load_xq = System.Load.xq;
      T1_idle = Controller.idle;
      T1_wait = Controller.wait;
      T1_action = Controller.action;
      G1_up_limit = Controller.up_limit;
      
    end PrimaryControlledSystem;
    
    model ClosedLoop 
      "Test case power system with example emergency control applied" 
      parameter Integer strategy=0;
      parameter Real FaultTime=100;
      parameter Real TimeOfControl=110;
      output Real V2;
      output Real V3;
      output Real V4;
      output Real Efd;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.49, 
          y=0.02, 
          width=0.44, 
          height=0.65), 
        Documentation(info="This model illustrates three possibilities of
emergency control. The aim of the emergeny control is
to stabilize the voltage of Bus 4 at a value higher than 0.9 
p.u.

It is trivial to design emergency control strategies
that will arrest the voltage collapse for a simple
with this simple structure based on heuristics.

strategy=0:
No emergency control, the voltage collapses after 568.5.

strategy=1:
Attempt to stabilize by change of transformer voltage control
setpoint, 

strategy=2:
Stabilitation by capacitor switching

strategy=3:
Stabilization by capacitor swithcing and tap reference
change

strategy=4:
Stabilization by load shedding


"));
      ABB_CC_Testcase.Fournode.PrimaryControlledSystem PCS annotation (extent=[
            -30, 48; -10, 68]);
    equation 
      // control strategy
      if strategy == 0 then
        // no emergency control
        PCS.TapVref = 1;
        PCS.CapStep = 2;
        PCS.LoadStep = 0;
      elseif strategy == 1 then
        //attempt to stabilize by tap reference change
        PCS.TapVref = if time > TimeOfControl then 0.92 else 1;
        PCS.CapStep = 2;
        PCS.LoadStep = 0;
      elseif strategy == 2 then
        //attempt to stabilize by cap switching
        PCS.TapVref = 1;
        PCS.CapStep = if time > TimeOfControl then 3 else 2;
        PCS.LoadStep = 0;
      elseif strategy == 3 then
        //succesful stabilization by cap switching and tap reference change
        PCS.TapVref = if time > TimeOfControl then 0.92 else 1;
        PCS.CapStep = if time > TimeOfControl then 3 else 2;
        PCS.LoadStep = 0;
      elseif strategy == 4 then
        // stabilization by load shedding
        PCS.TapVref = 1;
        PCS.CapStep = 2;
        PCS.LoadStep = if time > TimeOfControl then 3 else 0;
      end if;
      PCS.faulted = if time > FaultTime then true else false;
      V2 = PCS.System.V2;
      V3 = PCS.System.V3;
      V4 = PCS.System.V4;
      Efd = PCS.Controller.Efd;
    end ClosedLoop;
  end Fournode;
  

end ABB_CC_Testcase;
